/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _MOZCELLML_H
#define _MOZCELLML_H

#include <mozilla-config.h>
#include <nsISimpleEnumerator.h>
#include <nsCOMPtr.h>
#include <nsIDocument.h>
#include <dom/nsIDOMXMLDocument.h>
#include <nsIDOMElement.h>
#include <nsIDOMNodeList.h>
#include <nsIDOMText.h>
#include <nsInterfaceHashtable.h>
#include <nsIStringBundle.h>
#include <nsArray.h>
#include <nsCOMPtr.h>
#include "mozICellML.h"
#include <nspr.h>
#include <necko/nsIServerSocket.h>
#include <nsIAsyncOutputStream.h>
#include <nsIAsyncInputStream.h>

#ifdef ENABLE_LEAKTRACE
#define LEAKTRACE_CREATED(x) printf("Created a " #x "\n");
#define LEAKTRACE_DELETED(x) printf("Deleted a " #x "\n");
#else
#define LEAKTRACE_CREATED(x)
#define LEAKTRACE_DELETED(x)
#endif

#define CELLML_NS_1_0 NS_LITERAL_STRING("http://www.cellml.org/cellml/1.0#")
#define CELLML_NS_1_1 NS_LITERAL_STRING("http://www.cellml.org/cellml/1.1#")

#define MATHML_NS NS_LITERAL_STRING("http://www.w3.org/1998/Math/MathML")

nsresult FilterNodeListByType(nsIDOMNodeList* nodelist, nsIArray** oarray,
                              PRUint16 keepType);
nsresult FilterNodeListByType(nsIDOMNodeList* nodelist, nsIArray** oarray,
                              PRUint16 keepType, const nsAString& ns);
nsresult FilterNodeListByNameAndTypeNS(nsIDOMNodeList* nodelist, nsIArray** oarray,
                                       PRUint16 keepType,
                                       const nsAString& ns,
                                       const nsAString& name);
nsresult FilterNodeListByNameAndTypeNS2(nsIDOMNodeList* nodelist, nsIArray** oarray,
                                       PRUint16 keepType,
                                       const nsAString& ns1,
                                       const nsAString& ns2,
                                       const nsAString& name);


#ifdef USE_CELLML_API

#define USE_MOZILLA_DOM
// Kludge to prevent CellML_Namespaces.hpp from being included...
#define CELLML_NAMESPACES_HPP
#include "CellML.hpp"

#endif

class mozCellMLVariable;
class nsIMutableArray;

class mozCellML
  : public mozICellML
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_MOZICELLML

  mozCellML();
private:
  ~mozCellML();

  nsCOMPtr<nsIDOMDocument> doc;
  nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable> allVariables;
  nsCOMArray<mozCellMLVariable> differentials, simples, constants;
  nsCOMArray<mozCellMLVariable> order;
  nsCOMPtr<nsIMutableArray> allComponents;
  nsCOMPtr<nsIStringBundle> bundle;
  nsCOMPtr<nsIMutableArray> variableList;
  nsCString ctrans;

  nsresult FindVariableElements(nsIDOMElement* root);
  nsresult FindVariableElement(const nsAString&, nsIDOMElement*, mozCellMLIVariable**);
  nsresult InstallComponent(nsIDOMElement*,
                            nsAString& componentName);
  nsresult WriteDifferentialAssignment(PRFileDesc* prfd, PRUint32 i, mozCellMLVariable* v,
                                   nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable>&);
  nsresult WriteSimpleEvaluation(PRFileDesc* prfd, PRUint32 i, mozCellMLVariable* v,
                             nsInterfaceHashtable<nsStringHashKey,mozCellMLVariable>&);
  NS_IMETHODIMP ParseCommandLine(const char* data, char** args, PRUint32 maxargs);
  NS_IMETHODIMP FreeCommandLine(char **args);
  PRUint32 SearchDeeplyForEncapsulations(nsIDOMElement*,
                                         nsInterfaceHashtable<nsStringHashKey,
                                                              nsIDOMElement>&);
  nsresult ExpandEncapsulatedList(nsCOMArray<nsIDOMElement>&,
                                  nsInterfaceHashtable<nsStringHashKey,
                                                       nsIDOMElement>&,
                                  nsInterfaceHashtable<nsStringHashKey,
                                                       nsIDOMElement>&,
                                  nsIDOMElement*);
  nsresult HashAvailableComponents(nsInterfaceHashtable<nsStringHashKey,
                                                        nsIDOMElement>&,
                                   nsIDOMElement*);
  nsresult AddDirectImports(nsInterfaceHashtable<nsStringHashKey,
                                                 nsIDOMElement>&,
                            nsInterfaceHashtable<nsStringHashKey,
                                                 nsIDOMElement>&,
                            nsIDOMElement*,
                            nsCOMArray<nsIDOMElement>&
                           );
  nsresult AddRelevantConnections(nsIDOMElement*,
                                  nsInterfaceHashtable<nsStringHashKey,
                                                       nsIDOMElement>&);



#ifdef USE_CELLML_API
  CellML_Model model;
#endif

  nsCOMArray<nsIDOMElement> allConnections;

  PRBool localValidationError;
  nsString validationErrorMessage;
public:
  nsresult ValidationError(const nsAString& msg);
  const char* Translate(const char* name);

  nsIDOMElement* GetDocumentElement()
  {
    nsIDOMElement* d;
    doc->GetDocumentElement(&d);
    return d;
  }
  nsresult FetchImports(nsIMutableArray** a);
  nsresult ImportLoaded(nsIDOMElement*, nsIDOMDocument*);
};

// This is a utility class used by the integrator and fitter...
class WeakServerSocketListenerAdaptor
  : public nsIServerSocketListener
{
public:
  WeakServerSocketListenerAdaptor(nsIServerSocketListener* ss)
    : weakSock(getter_AddRefs(NS_GetWeakReference(ss)))
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSISERVERSOCKETLISTENER
private:
  nsWeakPtr weakSock;
  ~WeakServerSocketListenerAdaptor() {};
};

class WeakInputCallbackAdaptor
  : public nsIInputStreamCallback
{
public:
  WeakInputCallbackAdaptor(nsIInputStreamCallback* ss)
    : weakSock(getter_AddRefs(NS_GetWeakReference(ss)))
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSIINPUTSTREAMCALLBACK
private:
  nsWeakPtr weakSock;
  ~WeakInputCallbackAdaptor() {};
};

class WeakOutputCallbackAdaptor
  : public nsIOutputStreamCallback
{
public:
  WeakOutputCallbackAdaptor(nsIOutputStreamCallback* ss)
    : weakSock(getter_AddRefs(NS_GetWeakReference(ss)))
  {
  }

  NS_DECL_ISUPPORTS
  NS_DECL_NSIOUTPUTSTREAMCALLBACK
private:
  nsWeakPtr weakSock;
  ~WeakOutputCallbackAdaptor() {};
};

#endif // _MOZCELLML_H
