function Parameter(variable, initial)
{
  this.variable = variable;
  this.initial = initial;
  return this;
}

Parameter.prototype =
{
  variable: null,
  initial: 0
};

function Measured(variable, bound, value)
{
  this.variable = variable;
  this.bound = bound;
  this.value = value;
  return this;
}

Measured.prototype =
{
  variable: null,
  bound: 0,
  value: 0
};

function SetupFitterWindow()
{
  window.model = window.arguments[0];
  window.bound = window.arguments[1];
  window.boundunit = window.arguments[2];
  window.variables = window.arguments[3];
  window.nvariables = window.arguments[4];
  var lh = document.getElementById("fitter-measurementbound");
  lh.setAttribute("label", window.bound + "(" + window.boundunit + ")");
  lh = document.getElementById("fitter-newmeasurement-boundunits");
  lh.setAttribute("value", window.boundunit);
  lh = document.getElementById("fitter-newmeasurement-bound-label");
  lh.setAttribute("value", window.bound + ": ");

  lh = document.getElementById("start-bound");
  lh.setAttribute("value", "Start " + window.bound);
  lh = document.getElementById("bound-units");
  lh.setAttribute("value", window.boundunit);

  var param_mp = document.getElementById("fitter-newparamvar-popup");
  var param_list = document.getElementById("fitter-paramlist");
  var measure_mp = document.getElementById("fitter-newmeasurevar-popup");

  // Populate various boxes...
  for (var i = 0; i < window.nvariables; i++)
  {
    var vt = variables[i].type;
    if (vt == Components.interfaces.mozCellMLIVariable.BOUND)
      ;
    else if (vt == Components.interfaces.mozCellMLIVariable.SIMPLE)
    {
      var mi = document.createElement("menuitem");
      mi.setAttribute("label", variables[i].name);
      mi.value = variables[i];
      measure_mp.appendChild(mi);
    }
    else if (vt == Components.interfaces.mozCellMLIVariable.DIFFERENTIAL)
    {
      var li = document.createElement("listitem");
      var lc1 = document.createElement("listcell");
      var lc2 = document.createElement("listcell");
      lc1.setAttribute("label", variables[i].name);
      var iv;
      try
      {
        iv = variables[i].initialValue;
      }
      catch (e)
      {
        iv = Math.random();
      }
      lc2.setAttribute("label", iv);
      li.appendChild(lc1);
      li.appendChild(lc2);
      li.value = new Parameter(variables[i], iv);
      param_list.appendChild(li);
      var mi = document.createElement("menuitem");
      mi.setAttribute("label", variables[i].name);
      mi.value = variables[i];
      measure_mp.appendChild(mi);
    }
    else if (vt == Components.interfaces.mozCellMLIVariable.CONSTANT)
    {
      var mi = document.createElement("menuitem");
      mi.setAttribute("label", variables[i].name);
      mi.value = variables[i];
      param_mp.appendChild(mi);
    }
  }
}

function CleanupFitterWindow()
{
  if (window.fitter != null)
  {
    window.fitter.destroy();
    window.fitter = null;
  }
}

function SelectedPotentialMeasurement()
{
  var menulist = document.getElementById("fitter-newmeasurevar-mlist");
  if (menulist.selectedItem == null)
    return;
  var variab = menulist.selectedItem.value;
  document.getElementById("fitter-newmeasurement-units")
          .setAttribute("value", variab.units);
}

function SelectedPotentialParameter()
{
  var menulist = document.getElementById("fitter-newparamvar-mlist");
  if (menulist.selectedItem == null)
    return;
  var variab = menulist.selectedItem.value;
  document.getElementById("fitter-newparam-estimate-units")
          .setAttribute("value", variab.units);
}

function AddSpecificMeasurement(variab, bound, value)
{
  var sb = document.getElementById("status-bar-display");
  var m = new Measured(variab, bound, value);
  var li = document.createElement("listitem");
  var lc1 = document.createElement("listcell");
  var lc2 = document.createElement("listcell");
  var lc3 = document.createElement("listcell");
  lc1.setAttribute("label", variab.name);
  lc2.setAttribute("label", bound);
  lc3.setAttribute("label", value);
  li.appendChild(lc1);
  li.appendChild(lc2);
  li.appendChild(lc3);
  li.value = m;
  var mbox = document.getElementById("fitter-measuredlist");
  mbox.appendChild(li);
  var strb = document.getElementById("mozcellml-string-bundle");
  sb.label = strb.getString("measurementAdded");
}

function AddMeasured()
{
  // Get the selected variable...
  var sb = document.getElementById("status-bar-display");
  var strb = document.getElementById("mozcellml-string-bundle");
  var menulist = document.getElementById("fitter-newmeasurevar-mlist");
  if (menulist.selectedItem == null)
  {
    sb.label = strb.getString("selectVariableFirst");;
    return;
  }
  var variab = menulist.selectedItem.value;
  var bound = document.getElementById("fitter-newmeasurement-bound").value;
  if (bound != parseFloat(bound))
  {
    sb.label = strb.getString("boundMustBeFloat");
    return;
  }
  var value = document.getElementById("fitter-newmeasurement-value").value;
  if (value != parseFloat(value))
  {
    sb.label = strb.getString("variableMustBeFloat");
    return;
  }

  AddSpecificMeasurement(variab, bound, value);
}

function BrowseLoadCSV()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  const nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
                     .createInstance(nsIFilePicker);
  fp.init(window, strb.getString("chooseMeasurementCSVLoad"),
          nsIFilePicker.modeOpen);
  fp.appendFilter(strb.getString("allCSV"), "*.csv; *.txt");
  fp.appendFilter("CSV", "*.csv");
  fp.appendFilter(strb.getString("textFiles"), "*.txt");
  fp.appendFilter(strb.getString("allFiles"), "*");
  if (fp.show() != nsIFilePicker.returnOK)
    return;
  var CSVField = document.getElementById("loadcsv-filename");
  CSVField.value = fp.file.path;
}

function LoadCSV()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var filename = document.getElementById("loadcsv-filename").value;
  var file = Components.classes["@mozilla.org/file/local;1"]
                    .createInstance(Components.interfaces.nsILocalFile);
  var stream;
  try
  {
    file.initWithPath(filename);
    stream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                       .createInstance(Components.interfaces.nsIFileInputStream);
    stream.init(file, 0x1 /* PR_RDONLY*/ , 420, 0);
  }
  catch (e)
  {
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("cantLoadFromPath");
    return;
  }
  try
  {
    var csvParser = Components.classes["@physiome.org/mozCellMLCSVParser;1"]
                              .createInstance(Components.interfaces
                                                        .mozCellMLICSVParser);
    csvParser.parseCSV(stream);
    var nr = csvParser.rowCount;
    var nv = new Object();
    var colnames = new Object();
    csvParser.fetchFields(nv, colnames);
    colnames = colnames.value;
    var variabs = new Array();
    var boundindex = -1;
    nv = nv.value;
    for (var i = 0; i < nv; i++)
    {
      variabs[i] = null;
      for (var j = 0; j < window.nvariables; j++)
        if (window.variables[j].name == colnames[i])
        {
          variabs[i] = window.variables[j];
          if (variabs[i].type ==
              Components.interfaces.mozCellMLIVariable.BOUND)
          {
            variabs[i] = null;
            boundindex = i;
          }
          break;
        }
    }
    if (boundindex == -1)
    {
      var statusBar = document.getElementById('status-bar-display');
      statusBar.label =
        strb.getFormattedString("cantFindCertainColumn", [window.bound]);
      return;
    }
    for (var i = 0; i < nr; i++)
    {
      nv = new Object();
      var data = new Object();
      csvParser.fetchData(i, nv, data);
      nv = nv.value;
      data = data.value;
      var bound = data[boundindex];
      if (isNaN(bound))
        continue;
      for (var j = 0; j < nv; j++)
      {
        if (variabs[j] == null || isNaN(data[j]))
          continue;
        AddSpecificMeasurement(variabs[j], bound, data[j]);
      }
    }
  }
  catch (e)
  {
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("Malformed CSV file.");
    return;
  }
}

function BrowseSaveCSV()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  const nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
                     .createInstance(nsIFilePicker);
  fp.init(window, strb.getString("chooseMeasurementCSVSave"),
          nsIFilePicker.modeSave);
  fp.appendFilter(strb.getString("allCSV"), "*.csv; *.txt");
  fp.appendFilter("CSV", "*.csv");
  fp.appendFilter(strb.getString("textFiles"), "*.txt");
  fp.appendFilter(strb.getString("allFiles"), "*");
  if (fp.show() == nsIFilePicker.returnCancel)
    return;
  var CSVField = document.getElementById("savecsv-filename");
  CSVField.value = fp.file.path;
}

function SwapMeasurements(mvars, mbound, mvals, idx1, idx2)
{
   var tmp = mvars[idx1];
   mvars[idx1] = mvars[idx2];
   mvars[idx2] = tmp;
   tmp = mbound[idx1];
   mbound[idx1] = mbound[idx2];
   mbound[idx2] = tmp;
   tmp = mvals[idx1];
   mvals[idx1] = mvals[idx2];
   mvals[idx2] = tmp;
}

function SortMeasurements(mvars, mbound, mvals, idx1, idx2)
{
   if (idx1 >= idx2)
     return;
   if (idx2 - idx1 == 1)
   {
     if (mbound[idx2] <= mbound[idx1])
       SwapMeasurements(mvars, mbound, mvals, idx1, idx2);
     return;
   }

   var mid = (idx1 + idx2) >> 1;
   var midBound = mbound[mid];
   SwapMeasurements(mvars, mbound, mvals, idx2, mid);
   mid = idx1;
   var i;
   for (i = idx1; i < idx2; i++)
     if (mbound[i] <= midBound)
       SwapMeasurements(mvars, mbound, mvals, i, mid++);
   SwapMeasurements(mvars, mbound, mvals, mid, idx2);
   if (mid > 0)
     SortMeasurements(mvars, mbound, mvals, idx1, mid - 1);
   SortMeasurements(mvars, mbound, mvals, mid + 1, idx2);
}

function StreamPrint(strm, strng)
{
  strng += "\r\n";
  strm.write(strng, strng.length);
}

function SaveCSV()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var filename = document.getElementById("savecsv-filename").value;
  var file = Components.classes["@mozilla.org/file/local;1"]
                    .createInstance(Components.interfaces.nsILocalFile);
  var stream;
  try
  {
    file.initWithPath(filename);
    stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                       .createInstance(Components.interfaces.nsIFileOutputStream);
    stream.init(
                file,
                 /* PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE */ 
                42,
                /*0644*/420,
                0
               );
  }
  catch (e)
  {
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("cantLoadFromPath");
    return;
  }

  pb = document.getElementById("fitter-measuredlist");
  pbc = pb.childNodes;

  // Find the list of variables being measured...
  var mvars = new Array();
  var allVars = new Array();
  var mbound = new Array();
  var mvals = new Array();
  
  var j = 0, i, l, k;
  for (i = 0, l = pbc.length; i < l; i++)
  {
    var node = pbc.item(i);
    if (node.localName != "listitem")
      continue;
    for (k = 0; k < allVars.length; k++)
      if (allVars[k] == node.value.variable)
        break;
    if (k == allVars.length)
      allVars[k] = node.value.variable;
    mvars[j] = k;
    mbound[j] = node.value.bound;
    mvals[j] = node.value.value;
    j++;
  }
  var nmeasurements = j;

  if (nmeasurements == 0)
  {
    stream.close();
    return;
  }

  var names = window.bound;
  for (i = 0; i < allVars.length; i++)
    names += "," + allVars[i].name;
  
  StreamPrint(stream, names);

  SortMeasurements(mvars, mbound, mvals, 0, mvals.length - 1);
  var i;
  for (i = 0; i < mbound.length;)
  {
    var bound = mbound[i];
    var values = new Array();
    var counts = new Array(), tot, passCount = 0;
    var isave = i;
    do
    {
      tot = 0;
      passCount++;
      i = isave;
      for (j = 0; j < allVars.length; j++)
      {
        counts[j] = 0;
        values[j] = NaN;
      }
      do
      {
        counts[mvars[i]]++;
        if (counts[mvars[i]] == passCount)
        {
          values[mvars[i]] = mvals[i];
          tot++;
        }
        i++;
      }
      while (i < mbound.length &&
             mbound[i] == bound);
      if (tot != 0)
      {
        var valueStr = bound + "";
        for (j = 0; j < allVars.length; j++)
          if (isNaN(values[j]))
            valueStr += ",";
          else
            valueStr += "," + values[j];
        StreamPrint(stream, valueStr);
      }
    }
    while (tot != 0);
  }

  stream.close();
  stream.close();
  var statusBar = document.getElementById('status-bar-display');
  statusBar.label = strb.getString("csvSaveComplete");
}

function DelMeasured()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var sb = document.getElementById("status-bar-display");
  var mbox = document.getElementById("fitter-measuredlist");
  var measured = mbox.selectedItem;
  if (measured == null)
  {
    sb.label = strb.getString("selectMeasurement");
    return;
  }
  mbox.removeChild(measured);
  measured = measured.value;
  var potentialMeasureVars = document.getElementById("fitter-newmeasurevar-popup");
  var pmvcn = potentialMeasureVars.childNodes;
  for (var i = 0; i < pmvcn.length; i++)
  {
    if (pmvcn.item(i).value != measured.variable)
      continue;
    var pmlist = document.getElementById("fitter-newmeasurevar-mlist");
    pmlist.selectedIndex = i;
    SelectedPotentialMeasurement();
    break;
  }
  document.getElementById("fitter-newmeasurement-bound").value = measured.bound;
  document.getElementById("fitter-newmeasurement-value").value = measured.value;
  sb.label = strb.getString("measurementRemoved");
}

function ClearMeasured()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  if (!confirm(strb.getString("confirmRemoveAllMeasurements")))
    return;
  var sb = document.getElementById("status-bar-display");
  var mbox = document.getElementById("fitter-measuredlist");
  var i = 0;
  var mboxc = mbox.childNodes;
  for (i = 0; i < mboxc.length; i++)
  {
    var mboxi = mboxc.item(i)
    if (mboxi.localName != "listitem")
      continue;
    mbox.removeChild(mboxi);
  }
  sb.label = strb.getString("clearedAllMeasurements");
}

function AddParameter()
{
  // Get the selected variable...
  var sb = document.getElementById("status-bar-display");
  var strb = document.getElementById("mozcellml-string-bundle");
  var menulist = document.getElementById("fitter-newparamvar-mlist");
  if (menulist.selectedItem == null)
  {
    sb.label = strb.getString("selectVariableFirst");
    return;
  }
  var variab = menulist.selectedItem.value;
  var iv = document.getElementById("fitter-newparam-estimate").value;
  if (iv != parseFloat(iv))
  {
    sb.label = strb.getString("estimateMustBeFloat");
    return;
  }

  if (menulist.selectedIndex > 0)
    menulist.selectedIndex--;
  else
    menulist.selectedItem = null;

  var p = new Parameter(variab, iv);
  var li = document.createElement("listitem");
  var lc1 = document.createElement("listcell");
  var lc2 = document.createElement("listcell");
  lc1.setAttribute("label", variab.name);
  lc2.setAttribute("label", iv);
  li.appendChild(lc1);
  li.appendChild(lc2);
  li.value = p;
  var mbox = document.getElementById("fitter-paramlist");
  mbox.appendChild(li);

  // Delete the option from the menulist...
  var menupopup = document.getElementById("fitter-newparamvar-popup");
  var mpc = menupopup.childNodes;
  for (var i = 0; i < mpc.length; i++)
  {
    var item = mpc.item(i);
    if (item.value != variab)
      continue;
    menupopup.removeChild(item);
    SelectedPotentialParameter();
    break;
  }

  sb.label = strb.getString("parameterAdded");
}

function DelParameter()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var sb = document.getElementById("status-bar-display");
  var mbox = document.getElementById("fitter-paramlist");
  var parameter = mbox.selectedItem;
  if (parameter == null)
  {
    sb.label = strb.getString("selectParameter");
    return;
  }
  mbox.removeChild(parameter);
  parameter = parameter.value;

  var potentialParamVars = document.getElementById("fitter-newparamvar-popup");

  var mi = document.createElement("menuitem");
  mi.setAttribute("label", parameter.variable.name);
  mi.value = parameter.variable;
  potentialParamVars.appendChild(mi);

  var npv = document.getElementById("fitter-newparamvar-mlist");
  npv.selectedIndex = potentialParamVars.childNodes.length - 1;
  document.getElementById("fitter-newparam-estimate").value = parameter.initial;
  sb.label = strb.getString("parameterRemoved");
}

function ChangeFitterButtons(d, fitter, run)
{
  var expert = 0;
  try
  {
    var prefservice =
      Components.classes["@mozilla.org/preferences-service;1"]
               .getService(Components.interfaces.nsIPrefService);
    var cellBranch = prefservice.getBranch("physiome.mozcellml.");
    expert = cellBranch.getIntPref("expertlevel");
  }
  catch (e) {}

  SetVisibility(d.getElementById("compile-fitter"), !fitter);
  SetVisibility(d.getElementById("kill-fitter"), fitter && (expert > 1));
  SetVisibility(d.getElementById("start-fit"), (!run) && fitter);
  SetVisibility(d.getElementById("cancel-fit"), run && fitter);
}

function CompileFitter()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  // Get a temporary file name, to store the C code in...
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIDirectoryServiceProvider);
  var p = new Object();
  var lf = dirService.getFile("TmpD", p)
                     .QueryInterface(Components.interfaces.nsILocalFile);
  lf.appendRelativePath("fitter.c");
  lf.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0600*/384);
  var cfile = lf.path;
  var lfx = dirService.getFile("TmpD", p)
                      .QueryInterface(Components.interfaces.nsILocalFile);
  if (navigator.platform.substring(0, 5) == "Win32")
    lfx.appendRelativePath("fitter.exe");
  else
    lfx.appendRelativePath("fitter");

  lfx.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
                  /*0700*/448);
  var xfile = lfx.path;

  var cc = GetCompiler();

  // XXX not all compilers take the same arguments...
  cc += " " + cfile + " -o " + xfile;

  var statusBar = document.getElementById('status-bar-display');

  statusBar.label = strb.getString("compilingFitter");
  ChangeFitterButtons(document, true, false);
  try
  {
    window.fitter = window.model.newFitter(cfile, cc, xfile);
    statusBar.label = strb.getString("fitterCompiled");
  }
  catch (e)
  {
    var valerr = window.model.checkValidity();
    if (valerr == "")
      statusBar.label = strb.getString("fittingFailed");
    else
    {
      statusBar.label =
        strb.getFormattedString("fittingFailedInvalidReason", valerr);
    }
    ChangeFitterButtons(document, false, false);
  }
  lf.remove(false);
}

function FitResultWatcher()
{
   return this;
}

FitResultWatcher.prototype =
{
  QueryInterface: function(aIID)
  {
    if (
        aIID.equals(Components.interfaces.mozCellMLIFitWatcher) ||
        aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference)
       )
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  done: function(varcount, params, values)
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("fittingComplete");
    if (window.fitter != null)
      ChangeFitterButtons(document, true, false);
    window.openDialog(
                      "chrome://mozcellml/content/modelfitted.xul",
                      "_blank",
                      "chrome,dialog=no",
                      window.model,
                      varcount,
                      params,
                      values
                     );
  },

  halted: function()
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    statusBar.label = strb.getString("fittingHalted");
    this.live = false;
    if (window.fitter != null)
      ChangeFitterButtons(document, true, false);
  },

  failed: function(emsg)
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    var statusBar = document.getElementById('status-bar-display');
    emsg = strb.getString(emsg);
    var err = strb.getFormattedString("fittingFailedReason", [emsg]);
    statusBar.label = err;
    alert(err);
    this.live = false;
    if (window.fitter != null)
      ChangeFitterButtons(document, true, false);
  },

  live: false,
  closed: false
};

function StartFitting()
{
  var pb = document.getElementById("fitter-paramlist");
  var pbc = pb.childNodes;
  var strb = document.getElementById("mozcellml-string-bundle");

  var sb = document.getElementById("status-bar-display");

  // Find the list of parameters...
  var paramvars = new Array();
  var paramvals = new Array();

  var i, l, j=0;

  for (i = 0, l = pbc.length; i < l; i++)
  {
    var node = pbc.item(i);
    if (node.localName != "listitem")
      continue;
    paramvars[j] = node.value.variable;
    paramvals[j] = node.value.initial;
    j++;
  }
  var nparameters = j;
  if (nparameters == 0)
  {
    sb.label = strb.getString("mustHaveParameter");
    return;
  }
  
  pb = document.getElementById("fitter-measuredlist");
  pbc = pb.childNodes;

  // Find the list of measurements...
  var mvars = new Array();
  var mbound = new Array();
  var mvals = new Array();
  
  j = 0;
  for (i = 0, l = pbc.length; i < l; i++)
  {
    var node = pbc.item(i);
    if (node.localName != "listitem")
      continue;
    mvars[j] = node.value.variable;
    mbound[j] = node.value.bound;
    mvals[j] = node.value.value;
    j++;
  }
  var nmeasurements = j;
  if (nmeasurements == 0)
  {
    sb.label = strb.getString("mustHaveMeasurement");
    return;
  }

  if (nmeasurements < nparameters)
  {
    sb.label = strb.getString("moreParamsMeasurements");
    return;
  }

  var startBound = document.getElementById("startx").value;
  if (startBound != parseFloat(startBound))
  {
    sb.label = strb.getString("ibMustBeFloat");
    return;
  }

  var relerr = document.getElementById("relerr").value;
  if (relerr != parseFloat(relerr))
  {
    sb.label = strb.getString("relErrMustBeFloat");
    return;
  }

  var abserr = document.getElementById("abserr").value;
  if (abserr != parseFloat(abserr))
  {
    sb.label = strb.getString("absErrMustBeFloat");
    return;
  }

  ChangeFitterButtons(document, true, true);

  var frw = new FitResultWatcher();
  window.fitter.registerWatcher(frw);
  window.frw = frw;

  try
  {
    window.fitter.start(
                        window.model,
                        startBound,
                        abserr,
                        relerr,
                        nparameters,
                        nmeasurements,
                        paramvars,
                        paramvals,
                        mvars,
                        mbound,
                        mvals
                       );
  }
  catch (e)
  {
    sb.label = strb.getString("fittingFailed");
    ChangeFitterButtons(document, true, false);
  }
}

function CancelFitting()
{
  if (window.fitter != null)
  {
    ChangeFitterButtons(document, true, false);
    window.fitter.suspend();
  }
}

function KillFitter()
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  if (window.fitter != null)
  {
    statusBar.label = strb.getString("killingFitter");
    window.fitter.destroy();
    window.fitter = null;
    ChangeFitterButtons(document, false, false);
    statusBar.label = strb.getString("fitterKilled");
  }
}

function SetupFitResultsWindow()
{
  window.model = window.arguments[0];
  window.nparameters = window.arguments[1];
  window.paramvars = window.arguments[2];
  window.paramvalues = window.arguments[3];

  var listbox = document.getElementById("fitter-paramlist");

  for (var i = 0; i < window.nparameters; i++)
  {
    var li = document.createElement("listitem");
    var lc = document.createElement("listcell");
    lc.setAttribute("label", window.paramvars[i].name);
    li.appendChild(lc);
    lc = document.createElement("listcell");
    lc.setAttribute("label", window.paramvalues[i]);
    li.appendChild(lc);
    li.value = window.paramvars[i];
    listbox.appendChild(li);
  }
}

function MakeFittedModel()
{
  var newmodel = Components.classes["@physiome.org/mozCellML;1"]
                  .createInstance(Components.interfaces.mozICellML);

  var newdom = window.model.dom.cloneNode(true)
                     .QueryInterface(Components.interfaces.nsIDOMDocument);
  newmodel.dom = newdom;
  newmodel.initialise();

  var i;
  for (i = 0; i < window.nparameters; i++)
  {
    newmodel.getVariable(window.paramvars[i].name,
                         window.paramvars[i].component)
            .initialValue = window.paramvalues[i];
  }
  window.openDialog(
                    "chrome://mozcellml/content/modelview.xul",
                    "_blank",
                    "chrome,dialog=no",
                    newmodel,
                    true
                   );
}
